##############################################################################
#
# lib_tix.tcl - tix widget support library
#
# Copyright (C) 1996-1997 Stewart Allen
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#
# Initializes this library
#
proc vTcl:widget:lib:lib_tix {args} {
    global vTcl
	#
    # see if we're running tixWish. if not, return
	#
    if {[info command tixNoteBookFrame] == ""} {
        return
    }
    foreach i {tixNoteBook tixLabelFrame tixComboBox} {
        image create photo "ctl_$i" \
            -file [file join $vTcl(VTCL_HOME) images icon_$i.gif]
        vTcl:toolbar_add $i $i ctl_$i ""
    }
}

#
# additional attributes to set on insert
#
set vTcl(tixNoteBook,insert)       "-width 50 -height 50"
set vTcl(tixLabelFrame,insert)     "-width 100 -height 10"
set vTcl(tixComboBox,insert)       ""

#
# add to valid class list
#
lappend vTcl(classes) Blt_barchart Blt_graph TixVResize
lappend vTcl(classes) TixNoteBook TixNoteBookFrame TixControl TixVTree
lappend vTcl(classes) TixVStack TixTree TixAppContext
lappend vTcl(classes) TixBalloon TixButtonBox TixComboBox TixControl
lappend vTcl(classes) TixDialogShell TixDirList TixDirTree
lappend vTcl(classes) TixDragDropContext TixExFileSelectBox
lappend vTcl(classes) TixExFileSelectDialog TixFileSelectBox
lappend vTcl(classes) TixFileSelectDialog TixFileEntry TixLabelEntry
lappend vTcl(classes) TixLabelFrame TixLabelWidget tixListNoteBook
lappend vTcl(classes) TixOptionMenu TixPanedWindow TixPopupMenu
lappend vTcl(classes) TixPrimitive TixResizeHandle TixScrolledHList
lappend vTcl(classes) TixScrolledListBox TixScrolledText TixScrolledWidget
lappend vTcl(classes) TixScrolledWindow TixSelect TixShell
lappend vTcl(classes) TixSimpleDialog tixStackWindow TixStatusBar
lappend vTcl(classes) TixStdButtonBox TixStdDialogShell

#
# define dump procedures for widget types
#
set vTcl(TixNoteBook,dump_opt)         vTcl:dump:TixNoteBook
set vTcl(TixNoteBookFrame,dump_opt)    vTcl:dump:TixNoteBookFrame
set vTcl(TixLabelFrame,dump_opt)       vTcl:dump_widget_opt
set vTcl(TixComboBox,dump_opt)         vTcl:dump_widget_opt

#
# define whether or not do dump children of a class
#
set vTcl(TixNoteBook,dump_children)         1
set vTcl(TixNoteBookFrame,dump_children)    0
set vTcl(TixLabelFrame,dump_children)       0
set vTcl(TixComboBox,dump_children)         0

#
# individual widget commands executed after insert
#
proc vTcl:widget:tixNoteBook:inscmd {target} {
    puts "called post-insert on $target"
    return "
        $target add test -anchor center -label Test;
    "
}

#
# per-widget action to take upon edit-mode double-click
#
proc vTcl:widget:tixNoteBook:dblclick {target} {
    #puts "called edit-mode double-click on $target"
}

proc vTcl:widget:tixLabelFrame:dblclick {target} {
    #puts "called edit-mode double-click on $target"
}

proc vTcl:widget:tixComboBox:dblclick {target} {
    #puts "called edit-mode double-click on $target"
}

#
# per-widget-class dump procedures
#
proc vTcl:dump:TixNoteBook {target basename} {
    global vTcl tk_version
	set result [vTcl:dump_widget_opt $target $basename]
    set entries [$target pages]
    foreach page $entries {
        set conf [$target pageconfigure $page]
        set pairs [vTcl:conf_to_pairs $conf ""]
        append result "$vTcl(tab)$basename add $page \\\n"
        append result "[vTcl:clean_pairs $pairs]\n"
    }
    return $result
}

proc vTcl:dump:TixNoteBookFrame {target basename} {
	set output ""
	foreach child [winfo children $target] {
		foreach inside [winfo children $child] {
			append output [vTcl:dump:widgets $inside]
		}
	}
	return $output
}

